import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

/**
 * <b>Programm:</b> WaveThread<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> displays the animation
 */
public class Painting extends Canvas{
  MonoWave _mw = null;
  int _width = 0;
  int _height= 0;
  long _turn = 0;
  long _starttime = 0;
  CalcThread _calcthread = null;
  BufferedImage _buffer = null;
  
  static final int TYPE_INT_RGB = 1; // for BufferedImage
  
  /**
   * Constructor
   */
  public Painting(MonoWave mw,CalcThread cl) {
    _mw = mw;
    _calcthread = cl;
  }

  /**
   * offscreen rendering
   */
  public void offpaint(){
    //set data from _calcthread:
    _buffer.setData(_calcthread.getRaster());

    // draw FPS
    Graphics2D g2d = (Graphics2D) _buffer.getGraphics();
    g2d.setPaint(Color.white);
    float fps = 1000.0f*(float)_turn / (float)(System.currentTimeMillis()-_starttime);
    g2d.drawString(fps+"fps",10,10);
  }

  /**
   * Overwrites original update and removes the screnn clearing as we use
   * doublebuffering by using offpaint()
   */
  public void update(Graphics g){
    paint(g);
  }
  
  /**
   * initializes the offscreenbuffer, calls offpaint() and paints the buffer
   */
  public void paint(Graphics g){
    // if resized, inform _calcthread, create new BufferedImages
    if( (_width != getWidth()) || (_height != getHeight()) ) {
      _width  = getWidth(); // new values
      _height = getHeight(); 
      _buffer = new BufferedImage(_width,_height, TYPE_INT_RGB );

      // for FPS
      _starttime = System.currentTimeMillis();
      _turn = 1;
      
      // set new values in _calcthread, same object
      _calcthread.setXY(_width,_height);
    
    }else {
      _turn++;
      // avoid overflow
      if(_turn == 50) {
        _starttime = System.currentTimeMillis();
        _turn = 1;
      }
      offpaint(); //renew global offscreenbuffer
      g.drawImage(_buffer,0,0,this);
    }
  }
}
